from typing import Any, Dict, List
import pandas as pd
import yaml

# function and helper-functions to dynamicly extract nested dict paths: first to list of lists, then to a dict of dataframes and single values
def _into_rows(nested_dict: Dict[str, Any], path: List[Any], rows: List[List[Any]]):
    for key, value in nested_dict.items():
        innerpath = path + [key]
        if isinstance(value, dict):
            _into_rows(value, innerpath, rows)
        else:
            rows.append(innerpath + [value])

def _nested_dict_into_rows(nested_dict: Dict[str, Any]) -> List[List[Any]]:
    rows = []
    _into_rows(nested_dict, [], rows)
    return rows

def yaml_to_dict_of_dfs(nested_dict: Dict[str, Any]) -> Dict[str, Any]:
    full_out = {}
    for name, sub_config in nested_dict.items():
        if isinstance(sub_config, dict):
            full_out[name] = pd.DataFrame(_nested_dict_into_rows(sub_config))
            print(f"Added to 'data_all': a dataframe with {name=}")
        else:
            full_out[name] = {name: sub_config}
            print(f"Added to 'data_all': a key with {name=}")
    return full_out


def main():
    filepath = 'test_material_infos_coeff.yml'

    with open(filepath, "r") as file:
        raw_config = dict(yaml.safe_load(file))
        data_all = yaml_to_dict_of_dfs(raw_config)
        print(f'The content of the YAML-file {filepath=} is now accessible as "data_all"', '\n ')
    
    print(data_all)


if __name__ == "__main__":
    main()
